@@ -126,6 +126,10 @@ extern "C" {
 #  define MRN_HAVE_HA_EXTRA_SECONDARY_SORT_ROWID
 #endif
 
+#if MYSQL_VERSION_ID >= 100302 && defined(MRN_MARIADB_P)
+#  define MRN_HAVE_HA_EXTRA_PREPARE_FOR_ALTER_TABLE
+#endif
+
 #if MYSQL_VERSION_ID >= 50604 && !defined(MRN_MARIADB_P)
 #  define MRN_TIMESTAMP_USE_TIMEVAL
 #elif defined(MRN_MARIADB_P)
@@ -207,10 +211,6 @@ extern "C" {
 #  define MRN_HAVE_HTON_ALTER_TABLE_FLAGS
 #endif
 
-#if MYSQL_VERSION_ID >= 50706 && !defined(MRN_MARIADB_P)
-#  define MRN_FOREIGN_KEY_USE_CONST_STRING
-#endif
-
 #if MYSQL_VERSION_ID >= 100203 && defined(MRN_MARIADB_P)
 #  define MRN_FOREIGN_KEY_USE_METHOD_ENUM
 #endif
@@ -262,6 +262,10 @@ extern "C" {
 #  define MRN_HAVE_PSI_SERVER
 #endif
 
+#if MYSQL_VERSION_ID >= 100302 && defined(MRN_MARIADB_P)
+#  define MRN_HANDLER_HA_UPDATE_ROW_NEW_DATA_CONST
+#endif
+
 class ha_mroonga;
 
 /* structs */
@@ -359,6 +363,7 @@ private:
   grn_ctx *ctx;
   grn_obj *grn_table;
   grn_obj **grn_columns;
+  grn_column_cache **grn_column_caches;
   grn_obj **grn_column_ranges;
   grn_obj **grn_index_tables;
   grn_obj **grn_index_columns;
@@ -384,6 +389,9 @@ private:
   grn_table_cursor *empty_value_records_cursor;
   grn_obj *sorted_result;
   grn_obj *matched_record_keys;
+  grn_table_cursor *matched_record_keys_cursor;
+  grn_obj *condition_push_down_result;
+  grn_table_cursor *condition_push_down_result_cursor;
   String  *blob_buffers;
 
   // for error report
@@ -456,6 +464,7 @@ public:
 
   int delete_table(const char *name);
   int write_row(uchar *buf);
+  int update_row(const uchar *old_data, const uchar *new_data);
   int update_row(const uchar *old_data, uchar *new_data);
   int delete_row(const uchar *buf);
 
@@ -654,8 +663,13 @@ private:
 
   bool have_unique_index();
 
+  bool is_foreign_key_field(const char *table_name,
+                            const char *field_name,
+                            size_t field_name_length);
   bool is_foreign_key_field(const char *table_name,
                             const char *field_name);
+  bool is_foreign_key_field(const char *table_name,
+                            LEX_CSTRING &field_name);
 
   void push_warning_unsupported_spatial_index_search(enum ha_rkey_function flag);
   void clear_cursor();
@@ -768,6 +782,11 @@ private:
 #endif
   void storage_store_field_blob(Field *field,
                                 const char *value, uint value_length);
+#ifdef MRN_HAVE_MYSQL_TYPE_BLOB_COMPRESSED
+  void storage_store_field_blob_compressed(Field *field,
+                                           const char *value,
+                                           uint value_length);
+#endif
   void storage_store_field_geometry(Field *field,
                                     const char *value, uint value_length);
 #ifdef MRN_HAVE_MYSQL_TYPE_JSON
@@ -775,11 +794,14 @@ private:
                                 const char *value, uint value_length);
 #endif
   void storage_store_field(Field *field, const char *value, uint value_length);
+  void storage_get_column_value(int nth_column,
+                                grn_id record_id,
+                                grn_obj *value);
   void storage_store_field_column(Field *field, bool is_primary_key,
                                   int nth_column, grn_id record_id);
   void storage_store_fields(uchar *buf, grn_id record_id);
   void storage_store_fields_for_prep_update(const uchar *old_data,
-                                            uchar *new_data,
+                                            const uchar *new_data,
                                             grn_id record_id);
   void storage_store_fields_by_index(uchar *buf);
 
@@ -926,18 +948,18 @@ private:
                                               KEY *key_info,
                                               grn_obj *index_column);
   int storage_write_row_multiple_column_indexes(uchar *buf, grn_id record_id);
-  int storage_write_row_unique_index(uchar *buf,
+  int storage_write_row_unique_index(const uchar *buf,
                                      KEY *key_info,
                                      grn_obj *index_table,
                                      grn_obj *index_column,
                                      grn_id *key_id);
-  int storage_write_row_unique_indexes(uchar *buf);
+  int storage_write_row_unique_indexes(const uchar *buf);
   int wrapper_get_record_id(uchar *data, grn_id *record_id, const char *context);
-  int wrapper_update_row(const uchar *old_data, uchar *new_data);
-  int wrapper_update_row_index(const uchar *old_data, uchar *new_data);
-  int storage_update_row(const uchar *old_data, uchar *new_data);
-  int storage_update_row_index(const uchar *old_data, uchar *new_data);
-  int storage_update_row_unique_indexes(uchar *new_data);
+  int wrapper_update_row(const uchar *old_data, const uchar *new_data);
+  int wrapper_update_row_index(const uchar *old_data, const uchar *new_data);
+  int storage_update_row(const uchar *old_data, const uchar *new_data);
+  int storage_update_row_index(const uchar *old_data, const uchar *new_data);
+  int storage_update_row_unique_indexes(const uchar *new_data);
   int wrapper_delete_row(const uchar *buf);
   int wrapper_delete_row_index(const uchar *buf);
   int storage_delete_row(const uchar *buf);
@@ -1025,8 +1047,8 @@ private:
   int storage_ft_init();
   FT_INFO *wrapper_ft_init_ext(uint flags, uint key_nr, String *key);
   FT_INFO *storage_ft_init_ext(uint flags, uint key_nr, String *key);
-  void generic_ft_init_ext_add_conditions_fast_order_limit(
-      struct st_mrn_ft_info *info, grn_obj *expression);
+  void generic_ft_init_ext_add_conditions(struct st_mrn_ft_info *info,
+                                          grn_obj *expression);
   grn_rc generic_ft_init_ext_prepare_expression_in_boolean_mode(
     struct st_mrn_ft_info *info,
     String *key,
