@@ -26,10 +26,18 @@
 
 #ifdef MRN_ITEM_HAVE_ITEM_NAME
 #  define MRN_ITEM_FIELD_GET_NAME(item)        ((item)->item_name.ptr())
-#  define MRN_ITEM_FIELD_GET_NAME_LENGTH(item) ((item)->item_name.length())
+#  define MRN_ITEM_FIELD_GET_NAME_LENGTH(item)  \
+  (static_cast<int>(item->item_name.length()))
 #else
-#  define MRN_ITEM_FIELD_GET_NAME(item)        ((item)->name)
-#  define MRN_ITEM_FIELD_GET_NAME_LENGTH(item) (strlen((item)->name))
+#  ifdef MRN_ITEM_ITEM_NAME_IS_LEX_STRING
+#    define MRN_ITEM_FIELD_GET_NAME(item)        ((item)->name.str)
+#    define MRN_ITEM_FIELD_GET_NAME_LENGTH(item)        \
+  (static_cast<int>((item)->name.length))
+#  else
+#    define MRN_ITEM_FIELD_GET_NAME(item)        ((item)->name)
+#    define MRN_ITEM_FIELD_GET_NAME_LENGTH(item)        \
+  (static_cast<int>(strlen((item)->name)))
+#  endif
 #endif
 
 namespace mrn {
@@ -59,6 +67,11 @@ namespace mrn {
       {
         const Item_cond *cond_item = reinterpret_cast<const Item_cond *>(item);
         bool convertable = is_convertable(cond_item);
+        if (convertable) {
+          GRN_LOG(ctx_, GRN_LOG_DEBUG,
+                  "[mroonga][condition-push-down][true] "
+                  "convertable conditions");
+        }
         DBUG_RETURN(convertable);
       }
       break;
@@ -66,10 +79,20 @@ namespace mrn {
       {
         const Item_func *func_item = reinterpret_cast<const Item_func *>(item);
         bool convertable = is_convertable(func_item);
+        if (convertable) {
+          GRN_LOG(ctx_, GRN_LOG_DEBUG,
+                  "[mroonga][condition-push-down][true] "
+                  "convertable function condition: %u",
+                  func_item->functype());
+        }
         DBUG_RETURN(convertable);
       }
       break;
     default:
+      GRN_LOG(ctx_, GRN_LOG_DEBUG,
+              "[mroonga][condition-push-down][false] "
+              "unsupported top level conditionnot only one item: %u",
+              item->type());
       DBUG_RETURN(false);
       break;
     }
@@ -84,7 +107,12 @@ namespace mrn {
       DBUG_RETURN(false);
     }
 
-    if (cond_item->functype() != Item_func::COND_AND_FUNC) {
+    if (!(cond_item->functype() == Item_func::COND_AND_FUNC ||
+          cond_item->functype() == Item_func::COND_OR_FUNC)) {
+      GRN_LOG(ctx_, GRN_LOG_DEBUG,
+              "[mroonga][condition-push-down][false] "
+              "not AND nor OR conditions: %u",
+              cond_item->functype());
       DBUG_RETURN(false);
     }
 
@@ -118,9 +146,17 @@ namespace mrn {
         Item *left_item = arguments[0];
         Item *right_item = arguments[1];
         if (left_item->type() != Item::FIELD_ITEM) {
+          GRN_LOG(ctx_, GRN_LOG_DEBUG,
+                  "[mroonga][condition-push-down][false] "
+                  "left item of binary operation isn't field: %u",
+                  left_item->type());
           DBUG_RETURN(false);
         }
         if (!right_item->basic_const_item()) {
+          GRN_LOG(ctx_, GRN_LOG_DEBUG,
+                  "[mroonga][condition-push-down][false] "
+                  "right item of binary operation isn't constant: %u",
+                  right_item->type());
           DBUG_RETURN(false);
         }
 
@@ -144,12 +180,24 @@ namespace mrn {
         Item *min_item = arguments[1];
         Item *max_item = arguments[2];
         if (target_item->type() != Item::FIELD_ITEM) {
+          GRN_LOG(ctx_, GRN_LOG_DEBUG,
+                  "[mroonga][condition-push-down][false] "
+                  "target of BETWEEN isn't field: %u",
+                  target_item->type());
           DBUG_RETURN(false);
         }
         if (!min_item->basic_const_item()) {
+          GRN_LOG(ctx_, GRN_LOG_DEBUG,
+                  "[mroonga][condition-push-down][false] "
+                  "minimum value of BETWEEN isn't constant: %u",
+                  min_item->type());
           DBUG_RETURN(false);
         }
         if (!max_item->basic_const_item()) {
+          GRN_LOG(ctx_, GRN_LOG_DEBUG,
+                  "[mroonga][condition-push-down][false] "
+                  "maximum value of BETWEEN isn't constant: %u",
+                  max_item->type());
           DBUG_RETURN(false);
         }
 
@@ -159,6 +207,41 @@ namespace mrn {
                                  max_item);
         DBUG_RETURN(convertable);
       }
+    case Item_func::IN_FUNC:
+      if (!is_storage_mode_) {
+        DBUG_RETURN(false);
+      }
+      {
+        Item **arguments = func_item->arguments();
+        Item *target_item = arguments[0];
+        if (target_item->type() != Item::FIELD_ITEM) {
+          GRN_LOG(ctx_, GRN_LOG_DEBUG,
+                  "[mroonga][condition-push-down][false] "
+                  "target of IN isn't field: %u",
+                  target_item->type());
+          DBUG_RETURN(false);
+        }
+        Item_field *field_item = static_cast<Item_field *>(target_item);
+        uint n_arguments = func_item->argument_count();
+        for (uint i = 1; i < n_arguments; ++i) {
+          Item *value_item = arguments[i];
+          if (!value_item->basic_const_item()) {
+            GRN_LOG(ctx_, GRN_LOG_DEBUG,
+                    "[mroonga][condition-push-down][false] "
+                    "%uth value of IN isn't constant: %.*s: %u",
+                    i - 1,
+                    MRN_ITEM_FIELD_GET_NAME_LENGTH(field_item),
+                    MRN_ITEM_FIELD_GET_NAME(field_item),
+                    value_item->type());
+            DBUG_RETURN(false);
+          }
+        }
+
+        bool convertable = is_convertable_in(field_item,
+                                             arguments + 1,
+                                             n_arguments - 1);
+        DBUG_RETURN(convertable);
+      }
     default:
       DBUG_RETURN(false);
       break;
@@ -173,35 +256,59 @@ namespace mrn {
     Item_func::Functype func_type) {
     MRN_DBUG_ENTER_METHOD();
 
-    bool convertable = false;
-
     enum_field_types field_type = field_item->field->real_type();
     NormalizedType normalized_type = normalize_field_type(field_type);
     switch (normalized_type) {
     case STRING_TYPE:
-      if (value_item->type() == Item::STRING_ITEM &&
-          func_type == Item_func::EQ_FUNC) {
-        convertable = have_index(field_item, GRN_OP_EQUAL);
+      if (func_type == Item_func::EQ_FUNC &&
+          !have_index(field_item, GRN_OP_EQUAL)) {
+        GRN_LOG(ctx_, GRN_LOG_DEBUG,
+                "[mroonga][condition-push-down][false] "
+                "index for string equal operation doesn't exist: %.*s",
+                MRN_ITEM_FIELD_GET_NAME_LENGTH(field_item),
+                MRN_ITEM_FIELD_GET_NAME(field_item));
+        DBUG_RETURN(false);
       }
       break;
     case INT_TYPE:
       if (field_type == MYSQL_TYPE_ENUM) {
-        convertable = (value_item->type() == Item::STRING_ITEM ||
-                       value_item->type() == Item::INT_ITEM);
-      } else {
-        convertable = value_item->type() == Item::INT_ITEM;
+        if (value_item->type() != Item::STRING_ITEM &&
+            value_item->type() != Item::INT_ITEM) {
+          GRN_LOG(ctx_, GRN_LOG_DEBUG,
+                  "[mroonga][condition-push-down][false] "
+                  "constant value of enum binary operation "
+                  "isn't string nor integer: %.*s: %u",
+                  MRN_ITEM_FIELD_GET_NAME_LENGTH(field_item),
+                  MRN_ITEM_FIELD_GET_NAME(field_item),
+                  value_item->type());
+          DBUG_RETURN(false);
+        }
       }
       break;
     case TIME_TYPE:
-      if (is_valid_time_value(field_item, value_item)) {
-        convertable = have_index(field_item, func_type);
+      if (!is_valid_time_value(field_item, value_item)) {
+        GRN_LOG(ctx_, GRN_LOG_DEBUG,
+                "[mroonga][condition-push-down][false] "
+                "constant value of time binary operation "
+                "is invalid: %.*s: %u",
+                MRN_ITEM_FIELD_GET_NAME_LENGTH(field_item),
+                MRN_ITEM_FIELD_GET_NAME(field_item),
+                value_item->type());
+        DBUG_RETURN(false);
       }
       break;
     case UNSUPPORTED_TYPE:
+      GRN_LOG(ctx_, GRN_LOG_DEBUG,
+              "[mroonga][condition-push-down][false] "
+              "unsupported value of binary operation: %.*s: %u",
+              MRN_ITEM_FIELD_GET_NAME_LENGTH(field_item),
+              MRN_ITEM_FIELD_GET_NAME(field_item),
+              field_type);
+      DBUG_RETURN(false);
       break;
     }
 
-    DBUG_RETURN(convertable);
+    DBUG_RETURN(true);
   }
 
   bool ConditionConverter::is_convertable_between(const Item_field *field_item,
@@ -209,34 +316,144 @@ namespace mrn {
                                                   Item *max_item) {
     MRN_DBUG_ENTER_METHOD();
 
-    bool convertable = false;
-
-    enum_field_types field_type = field_item->field_type();
+    enum_field_types field_type = field_item->field->type();
     NormalizedType normalized_type = normalize_field_type(field_type);
     switch (normalized_type) {
     case STRING_TYPE:
-      if (min_item->type() == Item::STRING_ITEM &&
-          max_item->type() == Item::STRING_ITEM) {
-        convertable = have_index(field_item, GRN_OP_LESS);
+      if (!have_index(field_item, GRN_OP_LESS)) {
+        GRN_LOG(ctx_, GRN_LOG_DEBUG,
+                "[mroonga][condition-push-down][false] "
+                "index for string BETWEEN operation doesn't exist: %.*s",
+                MRN_ITEM_FIELD_GET_NAME_LENGTH(field_item),
+                MRN_ITEM_FIELD_GET_NAME(field_item));
       }
       break;
     case INT_TYPE:
-      if (min_item->type() == Item::INT_ITEM &&
-          max_item->type() == Item::INT_ITEM) {
-        convertable = have_index(field_item, GRN_OP_LESS);
+      if (field_type == MYSQL_TYPE_ENUM) {
+        if (min_item->type() != Item::STRING_ITEM &&
+            min_item->type() != Item::INT_ITEM) {
+          GRN_LOG(ctx_, GRN_LOG_DEBUG,
+                  "[mroonga][condition-push-down][false] "
+                  "minimum value of enum BETWEEN operation "
+                  "isn't string nor integer: %.*s: %u",
+                  MRN_ITEM_FIELD_GET_NAME_LENGTH(field_item),
+                  MRN_ITEM_FIELD_GET_NAME(field_item),
+                  min_item->type());
+          DBUG_RETURN(false);
+        }
+        if (max_item->type() != Item::STRING_ITEM &&
+            max_item->type() != Item::INT_ITEM) {
+          GRN_LOG(ctx_, GRN_LOG_DEBUG,
+                  "[mroonga][condition-push-down][false] "
+                  "maximum value of enum BETWEEN operation "
+                  "isn't string nor integer: %.*s: %u",
+                  MRN_ITEM_FIELD_GET_NAME_LENGTH(field_item),
+                  MRN_ITEM_FIELD_GET_NAME(field_item),
+                  max_item->type());
+          DBUG_RETURN(false);
+        }
       }
       break;
     case TIME_TYPE:
-      if (is_valid_time_value(field_item, min_item) &&
-          is_valid_time_value(field_item, max_item)) {
-        convertable = have_index(field_item, GRN_OP_LESS);
+      if (!is_valid_time_value(field_item, min_item)) {
+        GRN_LOG(ctx_, GRN_LOG_DEBUG,
+                "[mroonga][condition-push-down][false] "
+                "minimum value of time BETWEEN operation is invalid: "
+                "%.*s: %u",
+                MRN_ITEM_FIELD_GET_NAME_LENGTH(field_item),
+                MRN_ITEM_FIELD_GET_NAME(field_item),
+                min_item->type());
+        DBUG_RETURN(false);
+      }
+      if (!is_valid_time_value(field_item, max_item)) {
+        GRN_LOG(ctx_, GRN_LOG_DEBUG,
+                "[mroonga][condition-push-down][false] "
+                "maximum value of time BETWEEN operation is invalid: "
+                "%.*s: %u",
+                MRN_ITEM_FIELD_GET_NAME_LENGTH(field_item),
+                MRN_ITEM_FIELD_GET_NAME(field_item),
+                max_item->type());
+        DBUG_RETURN(false);
       }
       break;
     case UNSUPPORTED_TYPE:
+      GRN_LOG(ctx_, GRN_LOG_DEBUG,
+              "[mroonga][condition-push-down][false] "
+              "unsupported type of BETWEEN operation: %.*s: %u",
+              MRN_ITEM_FIELD_GET_NAME_LENGTH(field_item),
+              MRN_ITEM_FIELD_GET_NAME(field_item),
+              field_type);
+      DBUG_RETURN(false);
       break;
     }
 
-    DBUG_RETURN(convertable);
+    DBUG_RETURN(true);
+  }
+
+  bool ConditionConverter::is_convertable_in(const Item_field *field_item,
+                                             Item **value_items,
+                                             uint n_value_items) {
+    MRN_DBUG_ENTER_METHOD();
+
+    enum_field_types field_type = field_item->field->type();
+    NormalizedType normalized_type = normalize_field_type(field_type);
+
+    if (normalized_type == UNSUPPORTED_TYPE) {
+      GRN_LOG(ctx_, GRN_LOG_DEBUG,
+              "[mroonga][condition-push-down][false] "
+              "unsupported type of IN operation: %.*s: %u",
+              MRN_ITEM_FIELD_GET_NAME_LENGTH(field_item),
+              MRN_ITEM_FIELD_GET_NAME(field_item),
+              field_type);
+      DBUG_RETURN(false);
+    }
+
+    for (uint i = 0; i < n_value_items; ++i) {
+      Item *value_item = value_items[i];
+      switch (normalized_type) {
+      case STRING_TYPE:
+        if (!have_index(field_item, GRN_OP_EQUAL)) {
+          GRN_LOG(ctx_, GRN_LOG_DEBUG,
+                  "[mroonga][condition-push-down][false] "
+                  "index for string IN operation doesn't exist: %.*s",
+                  MRN_ITEM_FIELD_GET_NAME_LENGTH(field_item),
+                  MRN_ITEM_FIELD_GET_NAME(field_item));
+        }
+        break;
+      case INT_TYPE:
+        if (field_type == MYSQL_TYPE_ENUM) {
+          if (value_item->type() != Item::STRING_ITEM &&
+              value_item->type() != Item::INT_ITEM) {
+            GRN_LOG(ctx_, GRN_LOG_DEBUG,
+                    "[mroonga][condition-push-down][false] "
+                    "constant value of enum IN operation "
+                    "isn't string nor integer: %.*s: %u",
+                    MRN_ITEM_FIELD_GET_NAME_LENGTH(field_item),
+                    MRN_ITEM_FIELD_GET_NAME(field_item),
+                    value_item->type());
+            DBUG_RETURN(false);
+          }
+        }
+        break;
+      case TIME_TYPE:
+        if (!is_valid_time_value(field_item, value_item)) {
+          GRN_LOG(ctx_, GRN_LOG_DEBUG,
+                  "[mroonga][condition-push-down][false] "
+                  "%uth value of time IN operation is invalid: "
+                  "%.*s: %u",
+                  i,
+                  MRN_ITEM_FIELD_GET_NAME_LENGTH(field_item),
+                  MRN_ITEM_FIELD_GET_NAME(field_item),
+                  value_item->type());
+          DBUG_RETURN(false);
+        }
+        break;
+      default:
+        break;
+      }
+    }
+
+    DBUG_RETURN(true);
   }
 
   bool ConditionConverter::is_valid_time_value(const Item_field *field_item,
@@ -256,7 +473,7 @@ namespace mrn {
 
     bool error;
     Item *real_value_item = value_item->real_item();
-    switch (field_item->field_type()) {
+    switch (field_item->field->type()) {
     case MYSQL_TYPE_TIME:
       error = real_value_item->get_time(mysql_time);
       break;
@@ -317,6 +534,9 @@ namespace mrn {
       type = TIME_TYPE;
       break;
     case MYSQL_TYPE_VARCHAR:
+#ifdef MRN_HAVE_MYSQL_TYPE_VARCHAR_COMPRESSED
+    case MYSQL_TYPE_VARCHAR_COMPRESSED:
+#endif
       type = STRING_TYPE;
       break;
     case MYSQL_TYPE_BIT:
@@ -350,6 +570,9 @@ namespace mrn {
     case MYSQL_TYPE_MEDIUM_BLOB:
     case MYSQL_TYPE_LONG_BLOB:
     case MYSQL_TYPE_BLOB:
+#ifdef MRN_HAVE_MYSQL_TYPE_BLOB_COMPRESSED
+    case MYSQL_TYPE_BLOB_COMPRESSED:
+#endif
     case MYSQL_TYPE_VAR_STRING:
     case MYSQL_TYPE_STRING:
       type = STRING_TYPE;
@@ -455,44 +678,65 @@ namespace mrn {
     DBUG_RETURN(0);
   }
 
-  void ConditionConverter::convert(const Item *where, grn_obj *expression) {
+  void ConditionConverter::convert(const Item *where,
+                                   grn_obj *expression,
+                                   bool have_condition) {
     MRN_DBUG_ENTER_METHOD();
 
-    if (!where || where->type() != Item::COND_ITEM) {
+    if (!where) {
       DBUG_VOID_RETURN;
     }
 
-    Item_cond *cond_item = (Item_cond *)where;
-    List_iterator<Item> iterator(*((cond_item)->argument_list()));
+    switch (where->type()) {
+    case Item::COND_ITEM:
+      convert(static_cast<const Item_cond *>(where), expression, have_condition);
+      break;
+    case Item::FUNC_ITEM:
+      if (convert(static_cast<const Item_func *>(where), expression) &&
+          have_condition) {
+        grn_expr_append_op(ctx_, expression, GRN_OP_AND, 2);
+      }
+      break;
+    default:
+      break;
+    }
+
+    DBUG_VOID_RETURN;
+  }
+
+  bool ConditionConverter::convert(const Item_cond *cond_item,
+                                   grn_obj *expression,
+                                   bool have_condition) {
+    MRN_DBUG_ENTER_METHOD();
+
+    grn_operator logical_operator = GRN_OP_AND;
+    if (cond_item->functype() == Item_func::COND_OR_FUNC) {
+      logical_operator = GRN_OP_OR;
+    }
+
+    List<Item> *sub_item_list =
+      const_cast<Item_cond *>(cond_item)->argument_list();
+    List_iterator<Item> iterator(*sub_item_list);
     const Item *sub_item;
+    int n_conditions = have_condition ? 1 : 0;
+    bool added = false;
     while ((sub_item = iterator++)) {
       switch (sub_item->type()) {
+      case Item::COND_ITEM:
+        if (convert(static_cast<const Item_cond *>(sub_item),
+                    expression,
+                    n_conditions > 0)) {
+          added = true;
+          ++n_conditions;
+        }
+        break;
       case Item::FUNC_ITEM:
-        {
-          const Item_func *func_item = (const Item_func *)sub_item;
-          switch (func_item->functype()) {
-          case Item_func::EQ_FUNC:
-            convert_binary_operation(func_item, expression, GRN_OP_EQUAL);
-            break;
-          case Item_func::LT_FUNC:
-            convert_binary_operation(func_item, expression, GRN_OP_LESS);
-            break;
-          case Item_func::LE_FUNC:
-            convert_binary_operation(func_item, expression, GRN_OP_LESS_EQUAL);
-            break;
-          case Item_func::GE_FUNC:
-            convert_binary_operation(func_item, expression,
-                                     GRN_OP_GREATER_EQUAL);
-            break;
-          case Item_func::GT_FUNC:
-            convert_binary_operation(func_item, expression, GRN_OP_GREATER);
-            break;
-          case Item_func::BETWEEN:
-            convert_between(func_item, expression);
-            break;
-          default:
-            break;
+        if (convert(static_cast<const Item_func *>(sub_item), expression)) {
+          added = true;
+          if (n_conditions > 0) {
+            grn_expr_append_op(ctx_, expression, logical_operator, 2);
           }
+          ++n_conditions;
         }
         break;
       default:
@@ -500,12 +744,50 @@ namespace mrn {
       }
     }
 
-    DBUG_VOID_RETURN;
+    DBUG_RETURN(added);
   }
 
-  void ConditionConverter::convert_binary_operation(const Item_func *func_item,
+  bool ConditionConverter::convert(const Item_func *func_item,
+                                   grn_obj *expression) {
+    MRN_DBUG_ENTER_METHOD();
+
+    bool added = false;
+
+    switch (func_item->functype()) {
+    case Item_func::EQ_FUNC:
+      added = convert_binary_operation(func_item, expression, GRN_OP_EQUAL);
+      break;
+    case Item_func::LT_FUNC:
+      added = convert_binary_operation(func_item, expression, GRN_OP_LESS);
+      break;
+    case Item_func::LE_FUNC:
+      added = convert_binary_operation(func_item, expression, GRN_OP_LESS_EQUAL);
+      break;
+    case Item_func::GE_FUNC:
+      added = convert_binary_operation(func_item, expression,
+                                       GRN_OP_GREATER_EQUAL);
+      break;
+    case Item_func::GT_FUNC:
+      added = convert_binary_operation(func_item, expression, GRN_OP_GREATER);
+      break;
+    case Item_func::BETWEEN:
+      added = convert_between(func_item, expression);
+      break;
+    case Item_func::IN_FUNC:
+      added = convert_in(func_item, expression);
+      break;
+    default:
+      break;
+    }
+
+    DBUG_RETURN(added);
+  }
+
+  bool ConditionConverter::convert_binary_operation(const Item_func *func_item,
                                                     grn_obj *expression,
                                                     grn_operator _operator) {
+    MRN_DBUG_ENTER_METHOD();
+
     Item **arguments = func_item->arguments();
     Item *left_item = arguments[0];
     Item *right_item = arguments[1];
@@ -514,11 +796,13 @@ namespace mrn {
       append_field_value(field_item, expression);
       append_const_item(field_item, right_item, expression);
       grn_expr_append_op(ctx_, expression, _operator, 2);
-      grn_expr_append_op(ctx_, expression, GRN_OP_AND, 2);
+      DBUG_RETURN(true);
+    } else {
+      DBUG_RETURN(false);
     }
   }
 
-  void ConditionConverter::convert_between(const Item_func *func_item,
+  bool ConditionConverter::convert_between(const Item_func *func_item,
                                            grn_obj *expression) {
     MRN_DBUG_ENTER_METHOD();
 
@@ -544,9 +828,31 @@ namespace mrn {
 
     grn_expr_append_op(ctx_, expression, GRN_OP_CALL, 5);
 
-    grn_expr_append_op(ctx_, expression, GRN_OP_AND, 2);
+    DBUG_RETURN(true);
+  }
 
-    DBUG_VOID_RETURN;
+  bool ConditionConverter::convert_in(const Item_func *func_item,
+                                      grn_obj *expression) {
+    MRN_DBUG_ENTER_METHOD();
+
+    Item **arguments = func_item->arguments();
+    Item *target_item = arguments[0];
+
+    grn_obj *in_values_func = grn_ctx_get(ctx_, "in_values", -1);
+    grn_expr_append_obj(ctx_, expression, in_values_func, GRN_OP_PUSH, 1);
+
+    const Item_field *field_item = static_cast<const Item_field *>(target_item);
+    append_field_value(field_item, expression);
+
+    uint n_arguments = func_item->argument_count();
+    for (uint i = 1; i < n_arguments; ++i) {
+      Item *value_item = arguments[i];
+      append_const_item(field_item, value_item, expression);
+    }
+
+    grn_expr_append_op(ctx_, expression, GRN_OP_CALL, n_arguments);
+
+    DBUG_RETURN(true);
   }
 
   void ConditionConverter::append_field_value(const Item_field *field_item,
@@ -576,8 +882,9 @@ namespace mrn {
     case STRING_TYPE:
       grn_obj_reinit(ctx_, &value_, GRN_DB_TEXT, 0);
       {
+        String buffer;
         String *string;
-        string = const_item->val_str(NULL);
+        string = const_item->val_str(&buffer);
         GRN_TEXT_SET(ctx_, &value_, string->ptr(), string->length());
       }
       break;
